package kr.co.vividnext.sodalive.audio_content

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.gson.annotations.SerializedName
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.explorer.profile.GetAudioContentListResponse

class AudioContentViewModel(private val repository: AudioContentRepository) : BaseViewModel() {

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _audioContentListLiveData = MutableLiveData<GetAudioContentListResponse>()
    val audioContentListLiveData: LiveData<GetAudioContentListResponse>
        get() = _audioContentListLiveData

    private val _sort = MutableLiveData(Sort.NEWEST)
    val sort: LiveData<Sort>
        get() = _sort

    enum class Sort {
        @SerializedName("NEWEST")
        NEWEST,

        @SerializedName("PRICE_HIGH")
        PRICE_HIGH,

        @SerializedName("PRICE_LOW")
        PRICE_LOW
    }

    var isLast = false
    var page = 1
    private val size = 10

    fun getAudioContentList(userId: Long, onFailure: (() -> Unit)? = null) {
        if (!_isLoading.value!! && !isLast) {
            _isLoading.value = true
            compositeDisposable.add(
                repository.getAudioContentList(
                    id = userId,
                    page = page,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}",
                    sort = _sort.value!!
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success && it.data != null) {
                                if (it.data.items.isNotEmpty()) {
                                    page += 1
                                    _audioContentListLiveData.postValue(it.data!!)
                                } else {
                                    isLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.postValue(it.message)
                                } else {
                                    _toastLiveData.postValue(
                                        "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                    )
                                }

                                if (onFailure != null) {
                                    onFailure()
                                }
                            }

                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                            if (onFailure != null) {
                                onFailure()
                            }
                        }
                    )
            )
        }
    }

    fun changeSort(sort: Sort) {
        page = 1
        isLast = false
        _sort.postValue(sort)
    }
}
